﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive.ExperimentManager;
using HeuristicLab.Clients.Hive.Jobs;
using HeuristicLab.Clients.Hive.Views.ExperimentManager.TreeView;
using HeuristicLab.Core;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Optimization;

namespace HeuristicLab.Clients.Hive.Views.ExperimentManager {
  [View("HiveJob ItemTreeView")]
  //[Content(typeof(ItemCollection<HiveJob>), IsDefaultView = false)]
  [Content(typeof(ItemCollection<HiveJob>), IsDefaultView = false)]
  public partial class HiveJobItemTreeView : ItemTreeView<HiveJob> {
    public new ItemCollection<HiveJob> Content {
      get { return (ItemCollection<HiveJob>)base.Content; }
      set { base.Content = value; }
    }

    public HiveJobItemTreeView() {
      InitializeComponent();
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers on the Content here
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers on the Content here
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        // TODO: Put code here when content is null
      } else {
        // TODO: Put code here when content has been changed and is not null
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      // TODO: Put code here to enable or disable controls based on whether the Content is/not null or the view is ReadOnly
    }

    #region Event Handlers
    // TODO: Put event handlers here
    #endregion

    #region Child Control Events
    protected override void addButton_Click(object sender, EventArgs e) {
      IOptimizer optimizer = CreateItem<IOptimizer>();
      if (optimizer != null) {
        if (treeView.SelectedNode == null) {
          Content.Add(new OptimizerHiveJob(optimizer));
        } else {
          var experiment = ((HiveJob)treeView.SelectedNode.Tag).ItemJob.Item as Experiment;
          if (experiment != null) {
            experiment.Optimizers.Add(optimizer);
          }
        }
      }
    }

    protected override void removeButton_Click(object sender, EventArgs e) {
      if (treeView.SelectedNode != null) {
        var selectedItem = (HiveJob)treeView.SelectedNode.Tag;
        var parentItem = GetParentItem(selectedItem);
        if (parentItem == null) {
          Content.Remove((HiveJob)treeView.SelectedNode.Tag);
        } else {
          var experiment = parentItem.ItemJob.Item as Experiment;
          if (experiment != null) {
            experiment.Optimizers.Remove(((OptimizerJob)selectedItem.ItemJob).Item);
          }
        }        
      }
    }
    #endregion

    protected override ICollection<IItemTreeNodeAction<HiveJob>> GetTreeNodeItemActions(HiveJob selectedItem) {
      var actions = base.GetTreeNodeItemActions(selectedItem);
      if (selectedItem != null) {
        if (selectedItem.ItemJob.Item is Experiment) {

        }
        actions.Add(new DeleteJobTreeNodeAction(Content));
      }
      
      return actions;
    }
  }
}
